package com.trafficparrot.sdk.file;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.collect.ComparisonChain;
import com.trafficparrot.sdk.jms.Destination;
import com.trafficparrot.sdk.jms.MessageBodyType;

import static com.trafficparrot.sdk.jms.MessageBodyType.TEXT;
import static org.apache.commons.lang3.builder.ToStringBuilder.reflectionToString;
import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class FileMessageResponse implements Comparable<FileMessageResponse> {
    public static final String NO_TRANSFORMER = "NO_TRANSFORMER";

    public final Destination destination;
    public final String fileMessageResponseTransformerClassName;
    public final String fileName;
    public final String text;
    public final MessageBodyType bodyType;

    public FileMessageResponse(@JsonProperty("destination") Destination destination,
                               @JsonProperty("fileMessageResponseTransformerClassName") String fileMessageResponseTransformerClassName,
                               @JsonProperty("fileName") String fileName,
                               @JsonProperty("text") String text,
                               @JsonProperty("bodyType") MessageBodyType bodyType) {
        this.destination = destination;
        this.fileMessageResponseTransformerClassName = fileMessageResponseTransformerClassName;
        this.fileName = fileName;
        this.text = text;
        this.bodyType = bodyType;
    }

    public FileMessageResponse(Destination destination, String fileName, String text) {
        this(destination, NO_TRANSFORMER, fileName, text, TEXT);
    }

    @Override
    @JsonIgnore
    public String toString() {
        return reflectionToString(this, SHORT_PREFIX_STYLE);
    }

    @Override
    public int compareTo(FileMessageResponse other) {
        return ComparisonChain.start()
                .compare(destination, other.destination)
                .compare(text, other.text)
                .result();
    }

    public FileMessage createResponseMessage() {
        return new FileMessage(fileName, text, System.currentTimeMillis());
    }
}
