package com.trafficparrot.sdk.file;

import com.trafficparrot.sdk.jms.Destination;
import com.trafficparrot.sdk.jms.Slf4jTpLogger;
import com.trafficparrot.sdk.jms.TpLogger;

import static com.trafficparrot.sdk.file.FileMessageResponse.NO_TRANSFORMER;
import static java.lang.String.format;

public abstract class FileMessageResponseTransformer {
    protected final TpLogger logger;

    public FileMessageResponseTransformer() {
        this(new Slf4jTpLogger(FileMessageResponseTransformer.class));
    }

    public FileMessageResponseTransformer(TpLogger logger) {
        this.logger = logger;
    }

    public final FileMessageResponse transform(Destination requestDestination, FileMessage requestMessage, FileMessageResponse response) {
        try {
            FileMessageResponse fileMessageResponse = doTransform(requestDestination, requestMessage, response);
            if (!NO_TRANSFORMER.equals(fileMessageResponse.fileMessageResponseTransformerClassName)) {
                String errorMessage = format("Please set the fileMessageResponseTransformerClassName to %s in %s (it was set to %s)", NO_TRANSFORMER, getClass().getCanonicalName(), fileMessageResponse.fileMessageResponseTransformerClassName);
                logger.error(errorMessage);
                return new FileMessageResponse(response.destination, response.fileName, errorMessage);
            }
            return fileMessageResponse;
        } catch (Exception e) {
            String errorMessage = format("%s had a problem while transforming: %s", getClass().getCanonicalName(), e.getMessage());
            logger.error(errorMessage, e);
            return new FileMessageResponse(response.destination, response.fileName, errorMessage + ". See logs for more details.");
        }
    }

    protected abstract FileMessageResponse doTransform(Destination requestDestination, FileMessage requestMessage, FileMessageResponse response) throws Exception;

    public String getDisplayName() {
        return getClass().getSimpleName();
    }
}
