/*
 * Copyright (C) 2011 Thomas Akehurst
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This is a modification of HandlebarsXPathHelper.
 */
package com.trafficparrot.sdk.handlebars;

import com.github.jknack.handlebars.Options;
import com.github.tomakehurst.wiremock.common.Xml;
import com.github.tomakehurst.wiremock.extension.responsetemplating.helpers.HandlebarsXPathHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import static com.github.tomakehurst.wiremock.common.Exceptions.throwUnchecked;
import static javax.xml.xpath.XPathConstants.NODE;
import static org.w3c.dom.Node.ELEMENT_NODE;

// TODO(4/5) push to wiremock or handlebars java?
/**
 * This class mimics the behaviour of {@link HandlebarsXPathHelper} in order to extract a list of nodes.
 */
public class XPathList extends TrafficParrotHelper<String> {

    @Override
    public Object doApply(final String inputXml, final Options options) throws IOException {
        if (inputXml == null ) {
            return "{{ Warning: input XML parameter is missing }}";
        }

        if (options.param(0, null) == null) {
            return "{{ Warning: xpath parameter is missing }}";
        }

        final String xPathInput = options.param(0);

        Document doc;
        try (final StringReader reader = new StringReader(inputXml)) {
            InputSource source = new InputSource(reader);
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            final DocumentBuilder builder = factory.newDocumentBuilder();
            doc = builder.parse(source);
        } catch (SAXException se) {
            return "{{ Warning: '" + inputXml + "' is not valid XML }}";
        } catch (ParserConfigurationException e) {
            return throwUnchecked(e, Object.class);
        }

        try {
            final XPathFactory xPathfactory = XPathFactory.newInstance();
            final XPath xpath = xPathfactory.newXPath();

            Node node = (Node) xpath.evaluate(xPathInput, doc, NODE);

            if (node == null) {
                return "{{ Warning: '" + xPathInput + "' was not found }}";
            }

            NodeList childNodes = node.getChildNodes();
            int length = childNodes.getLength();
            List<String> renderedChildren = new ArrayList<>(length);
            for (int i = 0; i < length; i++) {
                Node item = childNodes.item(i);
                if (item.getNodeType() == ELEMENT_NODE) {
                    renderedChildren.add(Xml.toStringValue(item));
                }
            }
            return renderedChildren;
        } catch (XPathExpressionException e) {
            return "{{ Warning: '" + xPathInput + "' is not a valid XPath expression }}";
        }
    }
}
