package com.trafficparrot.sdk.http;

import com.github.tomakehurst.wiremock.common.FileSource;
import com.github.tomakehurst.wiremock.extension.Parameters;
import com.github.tomakehurst.wiremock.extension.ResponseTransformer;
import com.github.tomakehurst.wiremock.http.Request;
import com.github.tomakehurst.wiremock.http.Response;
import com.trafficparrot.sdk.jms.Slf4jTpLogger;
import com.trafficparrot.sdk.jms.TpLogger;

import static java.lang.String.format;
import static javax.servlet.http.HttpServletResponse.SC_INTERNAL_SERVER_ERROR;

public abstract class HttpResponseTransformer extends ResponseTransformer {
    protected final TpLogger logger;

    protected HttpResponseTransformer() {
        this.logger = new Slf4jTpLogger(HttpResponseTransformer.class);
    }

    @Override
    public final Response transform(Request request, Response response, FileSource fileSource, Parameters parameters) {
        try {
            return doTransform(request, response, fileSource, parameters);
        } catch (Exception e) {
            String errorMessage = format("%s had a problem while transforming: %s. See logs for more details.", getClass().getCanonicalName(), e.getMessage());
            logger.error(errorMessage, e);
            return Response.response().body(errorMessage).status(SC_INTERNAL_SERVER_ERROR).build();
        }
    }

    protected abstract Response doTransform(Request request, Response response, FileSource fileSource, Parameters parameters);

    @Override
    public String getName() {
        String name = getClass().getSimpleName();
        return name.substring(0, 1).toLowerCase() + name.substring(1);
    }

    @Override
    public boolean applyGlobally() {
        return false;
    }
}
