package com.trafficparrot.sdk.jms;

import com.fasterxml.jackson.core.JsonProcessingException;

import static com.trafficparrot.sdk.jms.JmsResponse.NO_TRANSFORMER;
import static com.trafficparrot.sdk.jms.MessageBodyType.*;
import static com.trafficparrot.sdk.messaging.Base64Representation.encodeBase64;
import static com.trafficparrot.sdk.messaging.ObjectJsonRepresentation.objectToJson;

public class JmsResponseBuilder {
    private Destination destination;
    private String text;
    private MessageBodyType bodyType;

    public JmsResponseBuilder like(JmsResponse response) {
        this.destination = response.destination;
        this.text = response.text;
        this.bodyType = response.bodyType;
        return this;
    }

    public JmsResponseBuilder withText(String text) {
        this.text = text;
        this.bodyType = TEXT;
        return this;
    }

    public JmsResponseBuilder withBytes(byte[] bytes) {
        this.text = encodeBase64(bytes);
        this.bodyType = BYTES;
        return this;
    }

    public JmsResponseBuilder withObject(Object object) throws JsonProcessingException {
        this.text = objectToJson(object);
        this.bodyType = javaClassType(object);
        return this;
    }

    public JmsResponseBuilder withDestination(Destination destination) {
        this.destination = destination;
        return this;
    }

    public JmsResponse build() {
        return new JmsResponse(destination, NO_TRANSFORMER, text, bodyType);
    }
}
