package com.trafficparrot.sdk.messaging;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static com.google.common.primitives.Bytes.concat;
import static java.util.Arrays.copyOfRange;

class FindAndReplaceBytes {
    public static byte[] findAndReplaceShortUtf(byte[] bytes, String find, String replace) throws IOException {
        return findAndReplaceBytes(bytes, utfBytes(find), utfBytes(replace));
    }

    public static byte[] findAndReplaceBytes(byte[] array, byte[] find, byte[] replace) {
        List<Integer> occurrences = findOccurrences(array, find);
        byte[][] pieces = new byte[occurrences.size() * 2 + 1][];
        int position = 0;
        for (int i = 0; i < occurrences.size(); i++) {
            int occurrence = occurrences.get(i);
            byte[] before = copyOfRange(array, position, occurrence);
            pieces[2 * i] = before;
            pieces[2 * i + 1] = replace;
            position = occurrence + find.length;
        }
        byte[] after = copyOfRange(array, position, array.length);
        pieces[occurrences.size() * 2] = after;
        return concat(pieces);
    }

    public static List<Integer> findOccurrences(byte[] array, byte[] search) {
        List<Integer> occurrences = new ArrayList<>();
        for (int i = 0; i < array.length - search.length + 1; i++) {
            if (occurs(array, i, search)) {
                occurrences.add(i);
                i += search.length - 1;
            }
        }
        return occurrences;
    }

    private static boolean occurs(byte[] array, int offset, byte[] search) {
        for (int j = 0; j < search.length; ++j) {
            if (array[offset + j] != search[j]) {
                return false;
            }
        }
        return true;
    }

    private static byte[] utfBytes(String string) throws IOException {
        try (ByteArrayOutputStream bytes = new ByteArrayOutputStream();
             DataOutputStream output = new DataOutputStream(bytes)) {
            output.writeUTF(string); // this is not the same as String.getBytes(StandardCharSets.UTF_8)
            return bytes.toByteArray();
        }
    }
}
